Package: mnirs
Title: Muscle Near-Infrared Spectroscopy Processing and Analysis
Version: 0.7.0
Authors@R: 
    person("Jem", "Arnold", , "jem.arnold@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-3908-9447"))
Description: Read, process, and analyse data from muscle near-infrared
    spectroscopy (mNIRS) devices. Import raw data from file and return
    time-series data and metadata. Standardised methods for cleaning,
    filtering, transforming, and analysing mNIRS data. Custom plot theme
    and colour palette. Intended for mNIRS researchers and practitioners
    in exercise physiology, sports science, and clinical practice.
License: MIT + file LICENSE
URL: https://jemarnold.github.io/mnirs/,
        https://github.com/jemarnold/mnirs
BugReports: https://github.com/jemarnold/mnirs/issues
Depends: R (>= 4.1)
Imports: cli, data.table, lifecycle, readxl, rlang, stats, tibble,
        tidyselect
Suggests: dplyr, ggplot2, knitr, quarto, scales, signal, testthat (>=
        3.0.0), zoo
VignetteBuilder: quarto
Config/Needs/website: quarto, tidyverse
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-08-01 04:13:09 UTC; Jem
Author: Jem Arnold [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-3908-9447>)
Maintainer: Jem Arnold <jem.arnold@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-01 05:10:07 UTC
Built: R 4.6.0; ; 2026-08-01 07:17:20 UTC; unix
