# Generated by roxygen2: do not edit by hand

S3method(plot,mnirs)
S3method(print,mnirs)
export(breaks_timespan)
export(by_label)
export(by_lap)
export(by_sample)
export(by_time)
export(create_mnirs_data)
export(example_mnirs)
export(extract_intervals)
export(filter_butter)
export(filter_butterworth)
export(filter_ma)
export(filter_mnirs)
export(filter_moving_average)
export(format_hmmss)
export(palette_mnirs)
export(read_mnirs)
export(replace_invalid)
export(replace_missing)
export(replace_mnirs)
export(replace_outliers)
export(resample_mnirs)
export(rescale_mnirs)
export(scale_color_mnirs)
export(scale_colour_mnirs)
export(scale_fill_mnirs)
export(shift_mnirs)
export(theme_mnirs)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(cli,col_blue)
importFrom(cli,qty)
importFrom(lifecycle,deprecated)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,check_installed)
importFrom(rlang,enquo)
importFrom(stats,coef)
importFrom(stats,median)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(tibble,tibble)
