#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom cli cli_abort
#' @importFrom cli cli_inform
#' @importFrom cli cli_warn
#' @importFrom cli col_blue
#' @importFrom cli qty
#' @importFrom lifecycle deprecated
#' @importFrom rlang !!
#' @importFrom rlang !!!
#' @importFrom rlang %||%
#' @importFrom rlang .data
#' @importFrom rlang :=
#' @importFrom rlang check_installed
#' @importFrom rlang enquo
#' @importFrom stats coef
#' @importFrom stats median
#' @importFrom stats rnorm
#' @importFrom stats setNames
#' @importFrom tibble tibble
## usethis namespace: end
NULL
