% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{group_intervals}
\alias{group_intervals}
\title{Apply grouping to intervals}
\usage{
group_intervals(
  df_list,
  nirs_channels,
  metadata,
  event_groups,
  zero_time = FALSE,
  verbose = TRUE
)
}
\description{
Apply grouping to intervals
}
\keyword{internal}
