% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{moxy_intervals.csv}
\alias{moxy_intervals.csv}
\title{0.5 Hz Moxy onboard export}
\format{
.csv file with seven columns and 936 rows:
\describe{
\item{mm-dd}{Recording date (dd-MMM format).}
\item{hh:mm:ss}{Recording time of day (hh:mm:ss format).}
\item{SmO2 Live}{Muscle oxygen saturation, raw signal (\%).}
\item{SmO2 Averaged}{Muscle oxygen saturation, rolling average (\%).}
\item{THb}{Total haemoglobin (arbitrary units).}
\item{Lap}{Lap marker (integer). Not typically in use.}
\item{Session Ct}{Session count of recordings.}
}

Channel mapping for \code{\link[=read_mnirs]{read_mnirs()}}:
\itemize{
\item \code{nirs_channels = c("SmO2 Live", "SmO2 Averaged", "THb")}
\item \code{time_channel = c("hh:mm:ss")}
\item \code{interval_times = list(start = c(124, 486, 848, 1210), end = c(364, 726, 1088, 1450))}
}
}
\source{
Moxy Monitor (Fortiori Design LLC), exported via Moxy Portal App.
(https://www.moxymonitor.com/)
}
\description{
Exported from Moxy onboard recording at 0.5 Hz
no smoothing. Containing four 4-minute cycling work intervals,
placed on the vastus lateralis muscle site.
}
\examples{
example_mnirs("moxy_intervals")

}
\seealso{
\code{\link[=read_mnirs]{read_mnirs()}}, \code{\link[=example_mnirs]{example_mnirs()}}
}
