% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{select_rename_data}
\alias{select_rename_data}
\title{Select data table columns and rename from channels, handling duplicates}
\usage{
select_rename_data(
  data,
  nirs_channels,
  time_channel,
  event_channel = NULL,
  keep_all = FALSE,
  verbose = TRUE
)
}
\description{
Select data table columns and rename from channels, handling duplicates
}
\keyword{internal}
