% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{detect_irregular_samples}
\alias{detect_irregular_samples}
\title{Report warnings for unbalanced time_channel samples}
\usage{
detect_irregular_samples(x, time_channel, verbose = TRUE)
}
\description{
Report warnings for unbalanced time_channel samples
}
\keyword{internal}
