% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{recycle_span}
\alias{recycle_span}
\title{recycle a single-element span to c(before, after)
positive -> c(0, x), negative -> c(x, 0)}
\usage{
recycle_span(span)
}
\description{
recycle a single-element span to c(before, after)
positive -> c(0, x), negative -> c(x, 0)
}
\keyword{internal}
