% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{zero_offset_data}
\alias{zero_offset_data}
\title{Recalculate time_channel values with zero offset at event time (t0)}
\usage{
zero_offset_data(data, time_channel, t0)
}
\description{
Recalculate time_channel values with zero offset at event time (t0)
}
\keyword{internal}
