% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{extract_df_list}
\alias{extract_df_list}
\title{Extract interval data by time range}
\usage{
extract_df_list(data, time_vec, interval_spec, nirs_channels)
}
\description{
Extract interval data by time range
}
\keyword{internal}
