% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnirs_methods.R
\name{print.mnirs}
\alias{print.mnirs}
\title{Methods for mnirs objects}
\usage{
\method{print}{mnirs}(x, ...)
}
\arguments{
\item{x}{Object of class \code{"mnirs"}.}

\item{...}{Additional arguments passed to \code{\link[=print]{print()}}.}
}
\value{
\item{\code{print}}{Returns \code{x} without class attributes.}
}
\description{
Generic methods for objects of class \code{"mnirs"}.
}
\examples{
x <- read_mnirs(
    example_mnirs("train.red"),
    nirs_channels = c(smo2 = "SmO2"),
    time_channel = c(time = "Timestamp (seconds passed)"),
    verbose = FALSE
) |>
    resample_mnirs(method = "linear", verbose = FALSE) |>
    extract_intervals(
        start = by_time(2452, 3168),
        span = c(-60, 120),
        verbose = FALSE
    )

print(x)

}
