% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{portamon-oxcap.xlsx}
\alias{portamon-oxcap.xlsx}
\title{10 Hz Artinis Oxysoft export recorded with Portamon}
\format{
.xlsx file with header metadata and six columns and 7943 rows:
\describe{
\item{Column 1}{Sample index (divide by sample rate for seconds).}
\item{Column 2}{tHb: total haemoglobin concentration change
(\eqn{\mu}M).}
\item{Column 3}{HHb: deoxyhaemoglobin concentration change (\eqn{\mu}M).}
\item{Column 4}{O2Hb: oxyhaemoglobin concentration change (\eqn{\mu}M).}
\item{Column 5}{Event marker (character).}
\item{Column 6}{Unmarked event label (character).}
}

Channel mapping for \code{\link[=read_mnirs]{read_mnirs()}}:
\itemize{
\item \code{nirs_channels = c(THb = 2, HHb = 3, O2Hb = 4)}
\item \code{time_channel = c(sample = 1)}
\item \code{event_channel = c(event = 5, label = "col_6")}
}
}
\source{
Artinis Medical Systems. Portamon, exported via Oxysoft desktop
software (https://artinis.com/)
}
\description{
Exported from Artinis Oxysoft, recorded on
Portamon at 10 Hz on the vastus lateralis muscle. Containing
two trials of repeated occlusion oxidative capacity testing,
each with 17 occlusions.
}
\examples{
example_mnirs("portamon")

}
\seealso{
\code{\link[=read_mnirs]{read_mnirs()}}, \code{\link[=example_mnirs]{example_mnirs()}}
}
