% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mnirs.R
\name{scale_colour_mnirs}
\alias{scale_colour_mnirs}
\alias{scale_color_mnirs}
\alias{scale_fill_mnirs}
\title{Scales for custom \emph{{mnirs}} palette}
\usage{
scale_colour_mnirs(..., aesthetics = "colour")

scale_color_mnirs(..., aesthetics = "colour")

scale_fill_mnirs(..., aesthetics = "fill")
}
\arguments{
\item{...}{Arguments passed to \code{ggplot2::discrete_scale()}.}

\item{aesthetics}{A character vector with aesthetic(s) passed to
\code{ggplot2::discrete_scale()}. \emph{Default} is \code{"colour"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2} scale object.
}
\description{
Scales for custom \emph{{mnirs}} palette
}
\examples{
\dontshow{if (rlang::is_installed(c("ggplot2", "scales"))) withAutoprint(\{ # examplesIf}
## plot example data
data <- read_mnirs(
    file_path = example_mnirs("moxy_ramp"),
    nirs_channels = c(smo2_left = "SmO2 Live",
                      smo2_right = "SmO2 Live(2)"),
    time_channel = c(time = "hh:mm:ss"),
    verbose = FALSE
)

ggplot2::ggplot(data, ggplot2::aes(x = time)) +
    theme_mnirs() +
    scale_colour_mnirs(name = NULL) +
    ggplot2::geom_line(ggplot2::aes(y = smo2_left, colour = "smo2_left")) +
    ggplot2::geom_line(ggplot2::aes(y = smo2_right, colour = "smo2_right"))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=theme_mnirs]{theme_mnirs()}}, \code{\link[=palette_mnirs]{palette_mnirs()}}
}
