% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{find_interval_time}
\alias{find_interval_time}
\title{resolve a single mnirs_interval object to time values}
\usage{
find_interval_time(
  interval,
  time_vec,
  event_vec = NULL,
  position = c("first", "last")
)
}
\description{
resolve a single mnirs_interval object to time values
}
\keyword{internal}
