% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{is_empty}
\alias{is_empty}
\title{Detect empty or NA strings}
\usage{
is_empty(x)
}
\description{
Detect empty or NA strings
}
\keyword{internal}
