% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{apply_span}
\alias{apply_span}
\title{apply span to resolved times and build interval_spec data frame}
\usage{
apply_span(interval_list, time_vec, span, verbose = TRUE)
}
\description{
apply span to resolved times and build interval_spec data frame
}
\keyword{internal}
