% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mnirs.R
\name{theme_mnirs}
\alias{theme_mnirs}
\title{Custom \emph{{mnirs}} ggplot2 theme}
\usage{
theme_mnirs(
  base_size = 14,
  base_family = "sans",
  border = c("partial", "full"),
  ink = "black",
  paper = "white",
  accent = "#0080ff",
  ...
)
}
\arguments{
\item{base_size}{Base font size, given in pts.}

\item{base_family}{Base font family.}

\item{border}{Define either a \emph{partial} or \emph{full} border around plots.}

\item{ink}{Colour for text and lines. \emph{Default} is \emph{"black"}.}

\item{paper}{Background colour. \emph{Default} is \emph{"white"}.}

\item{accent}{Accent colour for highlights. \emph{Default} is \emph{"#0080ff"}.}

\item{...}{Additional arguments to add to \verb{[ggplot2::theme()]}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2} theme object.
}
\description{
A \verb{[ggplot2][ggplot2::ggplot2-package]} theme for display.
}
\details{
\itemize{
\item \code{axis.title = element_text(face = "bold")} by \emph{default} Modify to \emph{"plain"}.
\item \code{panel.grid.major} & \code{panel.grid.major} set to blank. Modify to
\verb{= element_line()} for visible grid lines.
\item \code{legend.position = "top"} by \emph{default} Modify \code{"none"} to remove legend
entirely.
\item \code{border = "partial"} uses \code{panel.border = element_blank()} and
\code{axis.line = element_line()}.
\item \code{border = "full"} uses \verb{panel.border = element_rect(colour = "black",}
\verb{linewidth = 1)} and \code{axis.line = element_line()}.
\item \code{base_family = "sans"} by \emph{default}.
}
}
\examples{
\dontshow{if (rlang::is_installed(c("ggplot2", "scales"))) withAutoprint(\{ # examplesIf}
## plot example data
read_mnirs(
    file_path = example_mnirs("moxy_ramp"),
    nirs_channels = c(smo2_left = "SmO2 Live",
                      smo2_right = "SmO2 Live(2)"),
    time_channel = c(time = "hh:mm:ss"),
    verbose = FALSE
) |>
    plot(time_labels = TRUE)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=palette_mnirs]{palette_mnirs()}}, \code{\link[=scale_colour_mnirs]{scale_colour_mnirs()}}
}
