% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{moxy_ramp.xlsx}
\alias{moxy_ramp.xlsx}
\title{2 Hz PerfPro export of Moxy data}
\format{
.xlsx file with five columns and 2202 rows:
\describe{
\item{mm-dd}{Recording date (dd-MMM format).}
\item{hh:mm:ss}{Time of day (hh:mm:ss format).}
\item{SmO2 Live}{Muscle oxygen saturation, left leg (\%). Contains
simulated erroneous and missing samples.}
\item{SmO2 Live(2)}{Muscle oxygen saturation, right leg (\%).}
\item{Lap}{Lap marker (integer).}
}

Channel mapping for \code{\link[=read_mnirs]{read_mnirs()}}:
\itemize{
\item \code{nirs_channels = c("SmO2 Live", "SmO2 Live(2)")}
\item \code{time_channel = c("hh:mm:ss")}
\item \code{event_channel = c("Lap")}
\item \code{interval_times = list(start = c(204, 878))} (start and end of exercise)
}
}
\source{
Moxy Monitor (Fortiori Design LLC), exported via PerfPro Studio
desktop software (https://perfprostudio.com/).
}
\description{
Exported from PerfPro Studio software, recorded
at 0.5 Hz no smoothing and exported at 2 Hz. Containing a
ramp incremental cycling protocol, placed on bilateral
vastus lateralis muscle sites. Intentional data errors
(outliers, invalid values, and missing \code{NA} values) have
been introduced to demonstrate \emph{{mnirs}} cleaning
functions.
}
\examples{
example_mnirs("moxy_ramp")

}
\seealso{
\code{\link[=read_mnirs]{read_mnirs()}}, \code{\link[=example_mnirs]{example_mnirs()}}
}
