% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{artinis_intervals.xlsx}
\alias{artinis_intervals.xlsx}
\title{10 Hz Artinis Oxysoft export recorded with Oxymon MKIII}
\format{
.xlsx file with header metadata and five columns and 20919 rows:
\describe{
\item{Column 1}{Sample index (divide by sample rate for seconds).}
\item{Column 2}{O2Hb: oxyhaemoglobin concentration change (\eqn{\mu}M).}
\item{Column 3}{HHb: deoxyhaemoglobin concentration change (\eqn{\mu}M).}
\item{Column 4}{Event marker (character).}
\item{Column 5}{Unmarked event label (character).}
}

Channel mapping for \code{\link[=read_mnirs]{read_mnirs()}}:
\itemize{
\item \code{nirs_channels = c(O2Hb = 2, HHb = 3)}
\item \code{time_channel = c(sample = 1)}
\item \code{event_channel = c(event = 4, label = "col_5")}
\item \verb{interval_times = list(start = c(158, 999, 1750) end = c(493, 1333, 1961))} two intervals and post-exercise occlusion
}
}
\source{
Artinis Medical Systems. Oxymon MKIII, exported via Oxysoft desktop
software (https://artinis.com/)
}
\description{
Exported from Artinis Oxysoft, recorded on Oxymon MKIII at
50 Hz and exported at 10 Hz. Containing two 5-minute cycling work
intervals and an ischaemic occlusion, placed on the vastus lateralis
muscle site.
}
\examples{
example_mnirs("artinis_intervals")

}
\seealso{
\code{\link[=read_mnirs]{read_mnirs()}}, \code{\link[=example_mnirs]{example_mnirs()}}
}
