% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{as_mnirs_interval}
\alias{as_mnirs_interval}
\title{coerce raw values to mnirs_interval objects}
\usage{
as_mnirs_interval(x, arg = "start")
}
\arguments{
\item{x}{A raw value or mnirs_interval object.}

\item{arg}{Name of the argument for error messages.}
}
\description{
coerce raw values to mnirs_interval objects
}
\keyword{internal}
