% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_mnirs.R
\name{abort_validation}
\alias{abort_validation}
\title{validate_numeric abort message construction}
\usage{
abort_validation(name, integer = FALSE, msg1 = "", msg2 = "")
}
\description{
validate_numeric abort message construction
}
\keyword{internal}
