% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs.R
\name{example_mnirs}
\alias{example_mnirs}
\title{Get path to \emph{{mnirs}} example files}
\usage{
example_mnirs(file = NULL)
}
\arguments{
\item{file}{Name of file as character string. If \code{NULL}, returns a vector
of all available file names.}
}
\value{
A file path character string for selected example files stored in this
package.
}
\description{
Get path to \emph{{mnirs}} example files
}
\examples{
## lists all files
example_mnirs()

## partial matching will error if matches multiple
try(example_mnirs("moxy"))

example_mnirs("moxy_ramp")

}
