% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{rename_duplicates}
\alias{rename_duplicates}
\title{Rename duplicate strings in a vector with \code{make.unique()}}
\usage{
rename_duplicates(x)
}
\description{
Rename duplicate strings in a vector with \code{make.unique()}
}
\keyword{internal}
