% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnirs-package.R
\docType{package}
\name{mnirs-package}
\alias{mnirs}
\alias{mnirs-package}
\title{mnirs: Muscle Near-Infrared Spectroscopy Processing and Analysis}
\description{
Read, process, and analyse data from muscle near-infrared spectroscopy (mNIRS) devices. Import raw data from .csv or .xls(x) files and return time-series data and metadata. Includes standardised methods for cleaning, filtering, and pre-processing mNIRS data for subsequent analysis. Also includes a custom plot theme and colour palette. Intended for mNIRS researchers and practitioners in exercise physiology, sports science, and clinical rehabilitation with minimal coding experience required.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://jemarnold.github.io/mnirs/}
  \item \url{https://github.com/jemarnold/mnirs}
  \item Report bugs at \url{https://github.com/jemarnold/mnirs/issues}
}

}
\author{
\strong{Maintainer}: Jem Arnold \email{jem.arnold@gmail.com} (\href{https://orcid.org/0000-0003-3908-9447}{ORCID}) [copyright holder]

}
\keyword{internal}
