% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{preserve_metadata}
\alias{preserve_metadata}
\title{Zero-offset time values and add metadata}
\usage{
preserve_metadata(data, metadata, zero_time = FALSE)
}
\description{
Zero-offset time values and add metadata
}
\keyword{internal}
