% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\docType{data}
\name{datetime_formats}
\alias{datetime_formats}
\title{Datetime format strings for POSIXct parsing}
\format{
An object of class \code{character} of length 7.
}
\usage{
datetime_formats
}
\description{
Datetime format strings for POSIXct parsing
}
\keyword{internal}
