% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{parse_time_channel}
\alias{parse_time_channel}
\title{Parse time_channel character or dttm to numeric}
\usage{
parse_time_channel(
  data,
  time_channel,
  start_timestamp = NULL,
  add_timestamp = FALSE,
  zero_time = FALSE
)
}
\description{
Parse time_channel character or dttm to numeric
}
\keyword{internal}
