% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mnirs.R
\name{as_plot_data}
\alias{as_plot_data}
\title{Validate and bind a list of mnirs data frames for plotting}
\usage{
as_plot_data(x)
}
\description{
Validate and bind a list of mnirs data frames for plotting
}
\keyword{internal}
