% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{parse_sample_rate}
\alias{parse_sample_rate}
\title{Validate and Estimate Sample Rate}
\usage{
parse_sample_rate(
  data,
  file_header,
  time_channel,
  sample_rate = NULL,
  nirs_device = NULL,
  verbose = TRUE
)
}
\description{
Validate and Estimate Sample Rate
}
\keyword{internal}
