% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{recycle_param}
\alias{recycle_param}
\title{Recycle parameter to match number of events}
\usage{
recycle_param(param, n_events, event_groups, verbose = TRUE)
}
\description{
Recycle an argument vector to a list or repeat the last list item to match
the number of events.
}
\keyword{internal}
