% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{name_channels}
\alias{name_channels}
\title{Force names on character strings}
\usage{
name_channels(x)
}
\description{
Force names on character strings
}
\keyword{internal}
