% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{read_data_table}
\alias{read_data_table}
\title{Read data table from raw data}
\usage{
read_data_table(data, nirs_channels = NULL, header_row = 1L)
}
\description{
Read data table from raw data
}
\keyword{internal}
