% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{recycle_to_length}
\alias{recycle_to_length}
\title{Recycle parameter list to target length}
\usage{
recycle_to_length(param, n, name = c("event", "group"), verbose = TRUE)
}
\description{
Recycle parameter list to target length
}
\keyword{internal}
