% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{convert_type}
\alias{convert_type}
\title{Standardise comma decimals to periods in character columns}
\usage{
convert_type(data, time_channel, event_channel = NULL, verbose = TRUE)
}
\description{
Standardise comma decimals to periods in character columns
}
\keyword{internal}
