% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_mnirs.R
\name{make_list}
\alias{make_list}
\title{Validate if an item is a list}
\usage{
make_list(x)
}
\description{
Validate if an item is a list
}
\keyword{internal}
