% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_mnirs.R
\name{parse_channel_name}
\alias{parse_channel_name}
\title{Parse channel expressions for NSE}
\usage{
parse_channel_name(channel, data, env = rlang::caller_env())
}
\arguments{
\item{channel}{A quosure from \code{rlang::enquo()}.}

\item{data}{A data frame for tidyselect context.}

\item{env}{Environment for symbol evaluation.}
}
\value{
A character vector, list of character vectors, or \code{NULL}.
}
\description{
Converts quosures to character vectors, handling bare symbols, character
strings, lists, and tidyselect expressions.
}
\keyword{internal}
