% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mnirs.R
\name{palette_mnirs}
\alias{palette_mnirs}
\title{Custom \emph{{mnirs}} colour palette}
\usage{
palette_mnirs(...)
}
\arguments{
\item{...}{Either a single numeric specifying the number of colours to
return, or character strings specifying colour names. If empty, all
colours are returned.}
}
\value{
Named (when selecting by name) or unnamed character vector of
hex colours.
}
\description{
Custom \emph{{mnirs}} colour palette
}
\examples{
\dontshow{if (rlang::is_installed("scales")) withAutoprint(\{ # examplesIf}
scales::show_col(palette_mnirs())
scales::show_col(palette_mnirs(2))
scales::show_col(palette_mnirs("red", "orange"))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=theme_mnirs]{theme_mnirs()}}, \code{\link[=scale_colour_mnirs]{scale_colour_mnirs()}}
}
