% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs.R
\docType{data}
\name{mnirs_metadata}
\alias{mnirs_metadata}
\title{Metadata names  of class \code{"mnirs"}, retrieved with \code{attr()}}
\format{
An object of class \code{character} of length 8.
}
\usage{
mnirs_metadata
}
\description{
Metadata names  of class \code{"mnirs"}, retrieved with \code{attr()}
}
\keyword{internal}
