% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{train.red_intervals.csv}
\alias{train.red_intervals.csv}
\title{10 Hz Train.Red App export}
\format{
.csv file with header metadata and 10 columns and 11995 rows:
\describe{
\item{Timestamp (seconds passed)}{Elapsed time (s).}
\item{Lap/Event}{Lap number (numeric).}
\item{SmO2}{Muscle oxygen saturation, filtered (\%). Two channels have
duplicated names. If both are called, the second will be renamed to
\code{SmO2_1}.}
\item{SmO2 unfiltered}{Muscle oxygen saturation, raw signal (\%). Two
channels have duplicated names. If both are called, the second will
be renamed to \verb{SmO2 unfiltered_1}.}
\item{O2HB unfiltered}{Oxyhaemoglobin concentration, raw signal
(arbitrary units). Two channels have duplicated names. If both are
called, the second will be renamed to \verb{O2HB unfiltered_1}.}
\item{HHB unfiltered}{Deoxyhaemoglobin concentration, raw signal
(arbitrary units). Two channels have duplicated names. If both are
called, the second will be renamed to \verb{HHb unfiltered_1}.}
}

Channel mapping for \code{\link[=read_mnirs]{read_mnirs()}}:
\itemize{
\item \code{nirs_channels = c("SmO2", "SmO2 unfiltered", "O2HB unfiltered", "HHb unfiltered")}
\item \code{time_channel = c("Timestamp (seconds passed)")}
\item \code{event_channel = c("Lap/Event")}
\item \code{interval_times = list(start = c(2150.09, 2872.28), end = c(2452.26, 3167.98))}
\item \code{interval_times = list(start = c(65.94, 788.13), end = c(368.11, 1083.83))} from zero_time
}
}
\source{
Train.Red (Train.Red B.V.), exported via Train.Red app
(https://train.red/)
}
\description{
Exported from Train.Red app, recorded at 10 Hz.
Containing two 5-minute cycling work intervals, placed on
bilateral vastus lateralis muscle sites. Some data channels
have been omitted to reduce file size.
}
\examples{
example_mnirs("train.red")

}
\seealso{
\code{\link[=read_mnirs]{read_mnirs()}}, \code{\link[=example_mnirs]{example_mnirs()}}
}
