% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{detect_device_channels}
\alias{detect_device_channels}
\title{Detect known channels for a device}
\usage{
detect_device_channels(
  data,
  header_row = 1L,
  nirs_device = NULL,
  nirs_channels = NULL,
  time_channel = NULL,
  keep_all = FALSE,
  verbose = TRUE
)
}
\description{
Detect known channels for a device
}
\keyword{internal}
