% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{read_file}
\alias{read_file}
\title{Read raw data frame from file path}
\usage{
read_file(file_path)
}
\description{
Read raw data frame from file path
}
\keyword{internal}
