% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\docType{data}
\name{device_patterns}
\alias{device_patterns}
\title{Known channel names and detection patterns for supported mNIRS devices}
\format{
An object of class \code{list} of length 5.
}
\usage{
device_patterns
}
\description{
Known channel names and detection patterns for supported mNIRS devices
}
\keyword{internal}
