% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{detect_mnirs_device}
\alias{detect_mnirs_device}
\title{Detect mnirs device from file metadata}
\usage{
detect_mnirs_device(data)
}
\description{
Detect mnirs device from file metadata
}
\keyword{internal}
