% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{detect_time_channel}
\alias{detect_time_channel}
\title{Detect time_channel from header row}
\usage{
detect_time_channel(
  data,
  time_channel = NULL,
  nirs_device = NULL,
  verbose = TRUE
)
}
\description{
Detect time_channel from header row
}
\keyword{internal}
