% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{remove_empty_rows_cols}
\alias{remove_empty_rows_cols}
\title{Remove Empty Rows and Columns}
\usage{
remove_empty_rows_cols(data)
}
\description{
Remove Empty Rows and Columns
}
\keyword{internal}
