% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_interval_helpers.R
\name{ensemble_intervals}
\alias{ensemble_intervals}
\title{Ensemble average multiple intervals}
\usage{
ensemble_intervals(df_list, nirs_channels, metadata, verbose = TRUE)
}
\description{
Ensemble average multiple intervals
}
\keyword{internal}
