% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnirs_helpers.R
\name{extract_start_timestamp}
\alias{extract_start_timestamp}
\title{Extract earliest POSIXct value from file header metadata}
\usage{
extract_start_timestamp(file_header)
}
\description{
Extract earliest POSIXct value from file header metadata
}
\keyword{internal}
